classdef demo_partial_label_learning
% Timothee Cour, 30-Sep-2010 01:54:13 -- DO NOT DISTRIBUTE

    methods(Static)
        function scratch();
        demo_partial_label_learning.test_faces_tv_with_screenplay();
        demo_partial_label_learning.test_faces_tv();
        demo_partial_label_learning.test_fiw();
        end
        
        function test_faces_tv_with_screenplay();
        dir_base=demo_partial_label_learning.compute_paths();
        load(fullfile(dir_base,'lost_with_screenplay_supervision.mat'),'faces','labels','labels_bag');
        images={faces.image_registered};
        F=demo_partial_label_learning.compute_features_from_images(images);
        demo_partial_label_learning.test_ambiguous_learning(F,labels,labels_bag);
        end
        
        function test_faces_tv();
        dir_base=demo_partial_label_learning.compute_paths();
        load(fullfile(dir_base,'lost_season1_disc2_1.mat'),'movie');
        images={movie.faces.image_registered};
        [names,ignore,labels]=unique(vec({movie.faces.name}));
        F=demo_partial_label_learning.compute_features_from_images(images);
        demo_partial_label_learning.test_ambiguous_learning(F,labels);
        0;
        end
        
        function test_fiw();
        dir_base=demo_partial_label_learning.compute_paths();
        load(fullfile(dir_base,'fiw_data.mat'),'images','labels');
        F=demo_partial_label_learning.compute_features_from_images(images);
        demo_partial_label_learning.test_ambiguous_learning(F,labels);
        end
        
        function dir_base=compute_paths();
        %%change this to your path
        dir_base='/Users/timothee/research/data/hand_labels/annotated_movie_faces/tv_data';
%         dir_base='~/temp/ambiguous/tv_data/';
        end
        
        function test_ambiguous_learning(F,labels,labels_bag);
        %% compute random ambiguous labels
        if nargin<3
%             rand_funs.reset();temp=rand(1000,1);
            labels_bag=demo_partial_label_learning.compute_labels_bag_add(labels,2);
        end
        disp2('mean(sum(labels_bag,2))');
        disp2('1-1/mean(sum(labels_bag,2))');
        %% ambiguous learning: ours vs naive
        for i=1:2
            options.is_naive=i==1;            
            %% solve ambiguous labeling
            result=compute_labels_from_bags(F,labels_bag,[],options);

            %% evaluate result against groundtruth
            nb_error=density(labels~=result.labels);
            if options.is_naive
                method='naive';
            else
                method='ours ';
            end
            disp(['error rate with ',method, ': ',num2str(nb_error)]);
        end
        end
        
        function labels=compute_max_scores_from_labels_bags(scores,labels_bags);
        scores(~labels_bags)=-Inf;
        [ignore,labels]=max(scores,[],2);
        end
        
        function F=compute_features_from_images(images,k);
        %k: nb of PCA components 
        if nargin<2
            [p,q,r]=size(images{1});
            k=round(p*q/50); % (heuristic)
        end
        
        assert(iscell(images));
        for i=1:length(images)
            if size(images{i},3)>1
                images{i}=rgb2gray(images{i});
            end
        end
        X=cat(4,images{:});
        clear images
        [p,q,r,n]=size(X);
        X=reshape(X,p*q*r,n);
        X=im2double(X);
        X=X';
        
        F=computePCA(X,k,1);        
        %% display
        F=bsxfun(@plus,F,-mean(F,1));
        F=normalizeColumns_L2(F);
        end
        
        function labels_bag=compute_labels_bag_add(labels,nb_add);
        n=length(labels);
        L=max(labels);
        labels_bag=labels2mask(labels);
        for i=1:nb_add
            ind=randint2([n,1],L-i);
            for j=1:n
                ind0=find(labels_bag(j,:)==0);
                ind(j)=ind0(ind(j));
            end
            mask=labels2mask(ind);
            labels_bag=labels_bag | mask;
        end
        end
        
    end
end
