//#include "vector2.cpp"
//TODO:use memcpy, memset, etc (+use http://www.vik.cc/daniel/portfolio/memcpy.htm ?)
// Timothee Cour, 30-Sep-2010 01:54:13 -- DO NOT DISTRIBUTE

#pragma once

//#include <stdlib.h>
//#include <cstring>
//#include <mex.h>

//#define __VECTOR2_THROW_RANGE_ERRORS //TODO
#include "mex_util.cpp"


template<typename T>class vector2{
protected:
    T* _M_start;
    T* _M_finish;
    T* _M_end_of_storage;
    bool is_memory_persistent;
    
    T* _M_allocate(size_t __n){
		//		return (T*)(::operator new(__n*sizeof(T)));
        return (T*)malloc(__n);
        //TODO:safer alternative by checking for NULL (and avoiding losing ref to data)
        //can't use mxAlloc coz it's garbage collected so that would defeat purpose right?
    }
    void _M_deallocate(T* __p, size_t __n){
		//		::operator delete((void*)__p);
        free(__p);
    }
    T* _M_reallocate(T* __p, size_t __n){
        return (T*)realloc((void*)__p, __n*sizeof(T));
        //TODO:safer alternative by checking for NULL (and avoiding losing ref to data)
    }
    void _M_reallocate_smaller(T* __p, size_t __n){
//        mexPrintf();
//        DEBUG(__p);
//        DEBUG(__n);
        T*__p2=(T*)realloc((void*)__p, __n*sizeof(T));
        assert(!__n || __p2==__p);
    }
    
    inline void _initialize(size_t __n){
        _M_start = _M_allocate(__n);
        _M_finish = _M_start;
        _M_end_of_storage = _M_start + __n;
        is_memory_persistent=false;
    }
    
public:    
    vector2():_M_start(NULL), _M_finish(NULL), _M_end_of_storage(NULL), is_memory_persistent(false){
    }
    vector2(size_t __n){
        _initialize(__n);
        uninitialized_fill_to_end(_M_start, T());
    }
    vector2(size_t __n, const T& x)	{
        _initialize(__n);
        uninitialized_fill_to_end(_M_start, x);
    }
    ~vector2() {
        if(is_memory_persistent)        //TODO:assert _M_start has not been modified otherwise dangling pointers
            _M_reallocate_smaller(_M_start, _M_finish - _M_start);
        else
            cleanup();
    }
    
    void push_back(const T& x){
        if(_M_finish==_M_end_of_storage)
            increase_capacity();        
		*_M_finish++ = x;
    }
	void push_back() {
        if(_M_finish==_M_end_of_storage)
            increase_capacity();        
		*_M_finish++ = T();
	}
    void reserve(size_t n) {		
        if (capacity() < n)
			reserve_force(n);
    }
	
    void truncate() {
        _M_reallocate_smaller(_M_start, _M_finish - _M_start);
        _M_end_of_storage=_M_finish;
    }
    void set_persistent(){
        is_memory_persistent=true;
    }
	void reserve_resize(size_t n, const T& x) {
		reserve(n);
		_M_finish=_M_start+n;
	}
    void resize(size_t n, const T& x) {
		size_t n0=size();
        resize_uninitialized(n);
        if(n>n0){
            T*_M_finish_old=_M_start+n0;
            while(_M_finish_old!=_M_finish)
                *_M_finish_old++=x;
        }
    }
    void resize(size_t n) {
        resize(n, T());
    }
    void resize_uninitialized(size_t n) {		
        if(n>capacity()){
            size_t n0=size();
			size_t n2 = 2*n0;//capacity at least 2*n0
			if(n2<n)
				n2=n;
			_M_start=_M_reallocate(_M_start,n2);
			_M_end_of_storage=_M_start+n2;
		}            
        _M_finish=_M_start+n;
    }
    void clear() {
        _M_finish=_M_start;
    }
    void pop_back() {
        --_M_finish;
    }
    T& front() { return *_M_start; }
    const T& front() const { return *_M_start; }
    T& back() { return *(_M_finish - 1); }
    const T& back() const { return *(_M_finish - 1); }
    
    T* begin() { return _M_start; }
    const T* begin() const { return _M_start; }
    T* end() { return _M_finish; }
    const T* end() const { return _M_finish; }
    
    size_t size() const
    { return size_t(_M_finish - _M_start); }
    size_t max_size() const
    { return size_t(-1) / sizeof(T); }
    size_t capacity() const
    { return size_t(_M_end_of_storage - _M_start); }
    bool empty() const
    { return _M_start == _M_finish; }
    
    T& operator[](size_t __n) { return *(_M_start + __n); }
    const T& operator[](size_t __n) const { return *(_M_start + __n); }
    
	T& at(size_t n)	{ 
		_M_range_check(n); 
		return *(_M_start + n); 
	}
	const T& at(size_t n) const	{ 
		_M_range_check(n); return *(_M_start + n); 
	}
	
	
private:	
	void reserve_force(size_t n){
		size_t n1=size();
		_M_start=_M_reallocate(_M_start,n);
		_M_finish=_M_start+n1;
		_M_end_of_storage=_M_start+n;
	}
	void increase_capacity(){
		size_t n0=capacity();
		size_t n1=size();
		size_t n=n0 ? 2*n0 : 1;
		_M_start=_M_reallocate(_M_start,n);
		_M_end_of_storage=_M_start+n;	
		_M_finish=_M_start+n1;
	}
	/*
    void erase(T*p1, T*p2){
        //TODO
    }
    void insert(T*begin, T*end,const T& x){
        //TODO
    }
	 */
    void uninitialized_fill_to_end(T*start, T*end, const T& val){
        while(start != end)
            *start++=val;
        //assert
    }
    
	void _M_range_check(size_t __n) const {
//#ifdef __VECTOR3_THROW_RANGE_ERRORS
		if (__n >= size()){
			cleanup();
            assert(0); 
			mexErrMsgIdAndTxt("MATLAB:error","!! _M_range_check failed: line %d, function %s, file '%s', n=%llu,size=%llu\n", __LINE__,__func__,__FILE__,__n,this->size());
//			mexErrMsgIdAndTxt("MATLAB:error","!! _M_range_check '%s' failed: line %d, function %s, file '%s', n=%llu,size=%llu\n",#__n, __LINE__,__func__,__FILE__,__n,this->size());
		}
//#endif
	}
	
	void cleanup(){
		_M_deallocate(_M_start, _M_end_of_storage - _M_start);				
	}
	/*
	void 
	vector2<_Tp>::_M_insert_aux(iterator __position, const _Tp& __x)
	{
		if (_M_finish != _M_end_of_storage) {
			construct(_M_finish, *(_M_finish - 1));
			++_M_finish;
			_Tp __x_copy = __x;
			copy_backward(__position, _M_finish - 2, _M_finish - 1);
			*__position = __x_copy;
		}
		else {
			const size_type __old_size = size();
			const size_type __len = __old_size != 0 ? 2 * __old_size : 1;
			iterator __new_start = _M_allocate(__len);
			iterator __new_finish = __new_start;
			//		__STL_TRY {
			__new_finish = uninitialized_copy(_M_start, __position, __new_start);
			construct(__new_finish, __x);
			++__new_finish;
			__new_finish = uninitialized_copy(__position, _M_finish, __new_finish);
			//		}
			//		__STL_UNWIND((destroy(__new_start,__new_finish), 
			//					  _M_deallocate(__new_start,__len)));
			destroy(begin(), end());
			_M_deallocate(_M_start, _M_end_of_storage - _M_start);
			_M_start = __new_start;
			_M_finish = __new_finish;
			_M_end_of_storage = __new_start + __len;
		}
	}
	void 
	vector2<_Tp>::_M_insert_aux(iterator __position)
	{
		if (_M_finish != _M_end_of_storage) {
			construct(_M_finish, *(_M_finish - 1));
			++_M_finish;
			copy_backward(__position, _M_finish - 2, _M_finish - 1);
			*__position = _Tp();
		}
		else {
			const size_type __old_size = size();
			const size_type __len = __old_size != 0 ? 2 * __old_size : 1;
			iterator __new_start = _M_allocate(__len);
			iterator __new_finish = __new_start;
			//		__STL_TRY {
			__new_finish = uninitialized_copy(_M_start, __position, __new_start);
			construct(__new_finish);
			++__new_finish;
			__new_finish = uninitialized_copy(__position, _M_finish, __new_finish);
			//		}
			//		__STL_UNWIND((destroy(__new_start,__new_finish), 
			//					  _M_deallocate(__new_start,__len)));
			destroy(begin(), end());
			_M_deallocate(_M_start, _M_end_of_storage - _M_start);
			_M_start = __new_start;
			_M_finish = __new_finish;
			_M_end_of_storage = __new_start + __len;
		}
	}
	 */
};
